function [positions_new, orientations_new] = recalculateLinker(position_old, orientation_old, linkerRadius, numberOfStates)
% This function recalculates the dye positions around the contact point for
% every state od its temporal trajectory (ON/OFF states). It simply
% calculates a uniformly  distributed vectors on a full sphere, can not
% consider any information how the the linker is attached to the surface. 

% the original linker vector:
fi0=orientation_old(1);
theta0=orientation_old(2);
linkerVect_old=[...
    sin(theta0).*cos(fi0);...
    sin(theta0).*sin(fi0);...
    cos(theta0)]*linkerRadius;

% the position of the contact point:
contactPointPosition=position_old-linkerVect_old;

% the new random liner vectors:
theta1=acos(1-2*rand(1,numberOfStates));
fi1=2*pi*rand(1,numberOfStates);
linkerVect_new=[...
    sin(theta1).*cos(fi1);...
    sin(theta1).*sin(fi1);...
    cos(theta1)]*linkerRadius;

% the new dye positions and linker orientations:
positions_new=zeros(3, numberOfStates);
orientations_new=zeros(2, numberOfStates);
for idxState=1:numberOfStates
   positions_new(:, idxState)=contactPointPosition+linkerVect_new(:, idxState);
   orientations_new(:, idxState)=[fi1(idxState); theta1(idxState)];
end



end

