% adding the directories containing the required functions
run('../../addSubfoldersToPath.m')



%NA=[0.8, 0.9, 1.0, 1.1, 1.2, 1.3, 1.4];
NA=[1.4];
%NA=0.8:0.05:1.4;


PSF_params.mol.pos=[0,0,0];

% Parameters for 3D astigmatic simulation
PSF_params.meas.astigmatic_bool=false;
PSF_params.meas.tube_f=20;      %in "cm"
PSF_params.meas.magnification=100;
PSF_params.meas.cylindrical_f=10;   % in "m"

% Choose a PSF model
%PSF_params.psf.type='gaussian';
PSF_params.psf.type='scalar';
%PSF_params.psf.type='vectorial';
% Parameters for the polarization sensitive simulation
PSF_params.meas.polarization_bool=false;

if PSF_params.meas.polarization_bool && ~strcmp(PSF_params.psf.type, 'vectorial')
   error('Polarization sensitive simulation can only be performed with the vectorial PSF model.') 
end

%Parameters for the Gaussian PSf model
PSF_params.psf.colleff=0.3;

% Parameters for the scalar or for the vectorial PSF model
PSF_params.psf.focal_plane_dist=300;       % distance between the focal plane and the objective's aperture in um
PSF_params.psf.sample_depth=5;
PSF_params.psf.an_cone_angle=5;
PSF_params.psf.el_field_dir=[1;0;1i];
PSF_params.psf.an_tau_ratio=Inf;
PSF_params.psf.anisotropy_bool=false;
PSF_params.psf.readApertureFuncBool=true;

% Orientation of the fluorphores, for the vectorial PSF model
PSF_params.mol.orientation(1)=0*pi/180;       % fi (azimuth) angle of orientation of the dipole
PSF_params.mol.orientation(2)=0*pi/180;       % theta (polar) angle of orientation of the dipole

% dye parameters
PSF_params.dye.em_N_ofphotonpersec=5000;
PSF_params.dye.lambda=665;

% Acquisition parameters
PSF_params.aq.frame_size=10;
PSF_params.aq.pixel_size=160;
PSF_params.aq.NA=1.4;
PSF_params.aq.n_immersion=1.52;
PSF_params.aq.n_sample=1.33;

PSF_params.mol.ideal_PSF_bool=false;



for idxNA=1:numel(NA)
    
    %NA_LG=1.330490676098865e+00;    % NA=1.4
    %NA_LG=1.197249753359106e+00;    % NA=1.2
    
    PSF_params.aq.NA=NA(idxNA);
    
    
    [ NA_LG, delta_d_i ] = m_gen_PSF_numerical_calc_focal_position( PSF_params, false );
    NA_LG
    
    
    n_i=PSF_params.aq.n_immersion;
    n_s=PSF_params.aq.n_sample;
    d_s=PSF_params.psf.sample_depth*1E-6;
    
    
    l_max=60;        % the largest considered index of the binomial series of the Gibson and Lanni aberration
    j=transpose(0:l_max);
    
    %a=c+delta_t_i*d, where "a" contains the coefficients of the series of the Gibson and Lanni aberration;
    c=-(-1).^j*n_s*d_s.*(NA_LG/n_s).^(2*j).*binom_coeff(0.5, j);
    d=-(-1).^j*n_i.*(NA_LG/n_i).^(2*j).*binom_coeff(0.5, j);
    da=-(-1).^j*n_i.*(NA_LG/n_i).^(2*j).*binom_coeff(0.5, j);       % derivate of "a" with respect to "delta_t_i", equals to "d"
    
    c(1)=0;
    d(1)=0;
    da(1)=0;
    
    r=zeros(l_max,l_max);
    O=zeros(l_max,l_max);
    
    for x=1:l_max+1
        for y=x:l_max+1
            r(x,y)=(-1)^(y-x)*factorial(x+y-2)/(factorial(y-x)*factorial(x-1)*factorial(x-1));
        end
        O(x,x)=1/x;
    end
    
    q=inv(r);
    v=transpose(q*da)*O;
    w=v*q;
    
    minimize_Zernike_variance(idxNA, 1)=-w*c/(w*d);
    
    nullify_defocus(idxNA, 1)=-q(2,:)*c/(q(2,:)*d);
    paraxial(idxNA, 1)=-d_s*n_i/n_s;
    PSF_variance(idxNA, 1)=delta_d_i;      % NA=1.4
    %PSF_variance(idxNA, 1)=-6.190403687658389e-06;      % NA=1.2
    
    
    
end


figure()
plot(NA, minimize_Zernike_variance, NA, nullify_defocus, NA, paraxial, NA, PSF_variance)

A=[NA', paraxial, PSF_variance, minimize_Zernike_variance, nullify_defocus];

save('delta_d_i_methods_NA_1.4.dat', 'A', '-ascii', '-tabs')


% removing the directories added for the sscript
run('../../removeSubfoldersFromPath.m')


% function for calculating the generalized binoial coefficients of
% binomial series
    function [result]=binom_coeff(alpha, k)
        result=gamma(alpha+1)./(gamma(alpha-k+1).*factorial(k));
    end
