function [ NA_LG, delta_d_i ] = m_gen_PSF_numerical_calc_focal_position( params, testPSF_bool )
%function [  ] = m_gen_PSF_numerical_calc(  )
% This function calculates the Frauhofer diffraction of point source. First
% determines the number of pixels affected by the PSF, then calcultes the
% aperture function and calculates the diffraction integral numerically
% using pre-calculated matrices. Eventually calcultes the intensity
% distribution of the PSF and pixelises it.

% PARENT: m_gen_PSF_calc
% CHILD: m_gen_PSF_aperture_func, m_gen_PSF_DFT, m_gen_PSF_pixelise

% do not comment this:
numerical_calc_cont_bool=false;


%w_0_x w_0_y z_R_x z_R_y z_focal_x z_focal_y aperture_rad_obj aperture_rad_tube E

% these are for testing purposes, otherwise should be commented
% params.psf.type='scalar';
% %params.psf.type='vectorial';
% params.mol.pos=[0,0,000];
% params.dye.lambda=600;
% params.aq.frame_size=64;
% params.aq.pixel_size=160;
% params.aq.NA=1.4;
% params.dye.em_N_ofphotonpersec=100000;
% params.psf.colleff=0.4;
% params.psf.focal_plane_dist=300;       % distance between the focal plane and the objective's aperture in um
% params.psf.sample_depth=5;
% params.aq.n_immersion=1.518;        % refractive index of immersion medium
% params.aq.n_sample=1.331;          % refractive index of water at 650 nm
% params.psf.an_cone_angle=180;
% params.psf.el_field_dir=[1;1i;0];
% params.psf.an_tau_ratio=Inf;
% params.psf.anisotropy_bool=false;
% params.meas.polarization_bool=false;
% params.meas.astigmatic_bool=false;
% params.meas.tube_f=20;               % focal length of tube lens in cm
% params.meas.magnification=100;                     % lateral magnification of hte microscope
% params.meas.cylindrical_f=10;      % focal length of the cylindrical lens in m
% global DFT_matrix_bool
% global aperture_func_bool
% DFT_matrix_bool=false;
% aperture_func_bool=false;
% numerical_calc_cont_bool=false;
% params.mol.ideal_PSF_bool=false;
% params.mol.orientation(1)=0*pi/180;       % fi (azimuth) angle of orientation of the dipole
% params.mol.orientation(2)=90*pi/180;       % theta (polar) angle of orientation of the dipole

psf_type=params.psf.type;

% Booleans for controlling sampling point distribution which determines the
% numerical integration method. In case of "false" Euler integration is
% performed. In case of "true" a higher order numerical method is used, it
% currently has some bug which has yet to be found. Use these with "false".
params.numcalc.pixel_sampling_fill_bool=false;
params.numcalc.aperture_sampling_fill_bool=false;

% If "ideal" PSf calculation is performed. It is placed in the focal plane,
% in the middle of a pixel, active during the whole frame time and in case
% of vectorial PSF calculation avaraging is performed over all dipole
% orientation in order to get an "avarage" PSF and the polarization
% sensitive measurement is disable for this PSF.
% It is used for autofluorescent background level calculation and
% determining how much part of the intensity gets out of the active area
% after the DFT.
if params.mol.ideal_PSF_bool
    % place the molecule so it shall have the maximal possible PSF pixel values
    % ensure that the molecule is placed in the middle of a pixel
    if rem(params.aq.frame_size,2)==0
        params.mol.pos=[params.aq.pixel_size/2;params.aq.pixel_size/2;0];
    elseif rem(params.aq.frame_size,2)==1
        params.mol.pos=[0;0;0];
    else
        error('Something is wring with "frame_size" in "m_gen_trajs_to_seq.m"')
    end
    
    numerical_calc_cont_bool=false;         % recalculate some variables, should yet be false this case
    
    if strcmp(params.psf.type,'vectorial')
        params.psf.anisotropy_bool=true;    % anisotropic PSF
        params.meas.polarization_bool=false;    % no polarization sensitive measurement
        
        % let thesevariable have these values be e.g. linearily polarized
        % electric field, all possible dipole orientation, very fast
        % rotations compared to fluorescent lifetime, its dipole
        % orientation has uniform distribution during emissions
        params.mol.orientation=[0,0];
        params.psf.el_field_dir=[0;0;1];
        params.psf.an_cone_angle=180;
        params.psf.an_tau_ratio=Inf;
    end
end

% 128x128 grid seems enough, for an "ideal" scalar PSf it causes minimal
% numerical error (3040 vs 3036, later with 512x512 grid)
sampling_number_x=128;        % number of points (sampled points) in "x" dimensions in which the values of the aperture function and the PSF shall be calculated
sampling_index_x=1:sampling_number_x;   % indices used for the "x" sampled points
sampling_number_y=128;        % number of points (sampled points) in "y" dimensions in which the values of the aperture function and the PSF shall be calculated
sampling_index_y=1:sampling_number_y;   % indices used for the "y" sampled points

% positions of the molecule in the focal plane
x_pos=params.mol.pos(1)*1E-9;
y_pos=params.mol.pos(2)*1E-9;
% positions of the molecule on the optical axis
z_pos=params.mol.pos(3)*1E-9;        % "z" position of the molecule under design conditions in m

lambda=params.dye.lambda*1E-9;               % wavelength of emitted light in m

N_pixel_x=params.aq.frame_size;         % number of pixels on the frames in one dimension
N_pixel_y=params.aq.frame_size;         % number of pixels on the frames in one dimension
pixel_size=params.aq.pixel_size*1E-9;   % size of pixels in m
NA=params.aq.NA;                        % objective's numerical aperture
focal_plane_dist=params.psf.focal_plane_dist*1E-6;     % distante of the object from the objective's aperture under desing conditions in m
n_i=params.aq.n_immersion;                          % refractive index of immersion medium

polarization_bool=params.meas.polarization_bool;
anisotropy_bool=params.psf.anisotropy_bool;

% Calculating unchanging variables
if ~numerical_calc_cont_bool
    % approximating the PSF with Gaussian beam model
    [w_0_x, w_0_y, z_R_x, z_R_y, z_focal_x, z_focal_y]=m_gen_PSF_Gaussian_beam_params(params);

    % calculating the objective's aperture's radius from the NA, focal length
    % and the refractive index of immersion medium
    % the refractive index of the immersion medium and the cover slip are
    % assumed to be equal
    alpha_i=asin(NA/n_i);   %the maximal half-angle of the cone of light in the immersion medium

    % if the angles are complex, e.g. wrong values were given either for the
    % refractive indices or for the numerical aperture
    if ~isreal(alpha_i)
        error('Wrong immersion medium RI for the given numerical aperture.')
    end

    aperture_rad_obj=focal_plane_dist*tan(alpha_i);  % radius of the objective's aperture in m

    % conversion formulas between object and image space coordinates
    % "M_lateral" and "f_tube" terms drops out in the Fourier transfomation and phase front aberration, these will be omitted in the calculations
    % does not calculated:
        % M_lateral=n_i*f_tube/f_obj                    % -1*lateral magnification, gives the conversion between the object and image space coordinates
        % M_axial=M_lateral^2/n_s;                      % axial magnification, "-" is omitted because  the "z" axis point the same direction in the object and image space
        % mod_M_axial=1/n_s;                                 % axial magnification
    % calculated in "m_gen_PSF_numerical_calc.m": 
        % aperture_rad_tube=NA*f_tube/M_lateral;        % radius of the tube lens
        % mod_aperture_rad_tube=NA;                           % radius of the tube lens
    % calculated in "m_gen_PSF_aberration.m"
        % ap_coords_tube_x=x_tube_norm*aperture_rad_tube;       % "x" coordinates of sampled ponts on the tube lens' aperture
        % ap_coords_tube_y=y_tube_norm*aperture_rad_tube;       % "y" coordinates of sampled ponts on the tube lens' aperture
        % ap_coords_tube_ro_2=ro_tube_2*aperture_rad_tube^2;	% radius coordinates of sampled ponts on the tube lens' aperture
	% calculated in "m_gen_PSF_aberration.m"
        % x_pos_im=x_pos*M_lateral;                     % "x" position of the molecule in the image space
        % y_pos_im=y_pos*M_lateral;                     % "y" position of the molecule in the image space
        % z_pos_im=z_pos*M_axial;                       % "z" position of the molecule in the image space
        % tilt_aberration=-x_pos_im/f_tube*ap_coords_tube_x;            % optical path difference of the phase front tilt
        % mod_tilt_aberration=-x_pos*x_tube_norm*mod_aperture_rad_tube;                 % optical path difference of the phase front tilt
        % spherical_aberration=-0.5*ap_coords_tube_ro_2*z_pos_im/f_tube^2;              % optical path difference of the spherical aberration
        % mod_spherical_aberration=-0.5*ro_tube_2*z_pos*mod_aperture_rad_tube^2/n_s;     % optical path difference of the spherical aberration
    % calculated in "m_gen_PSF_DFT.m":
        % im_coords=d_x_image*(sampling_number_x-1)*((sampling_index_x-1)/(sampling_number_x-1)-0.5)*M_lateral;     % "x" position of the PSF's sampled points in the image plane
        % mod_im_coords=d_x_image*(sampling_number_x-1)*((sampling_index_x-1)/(sampling_number_x-1)-0.5);                 % "x" position of the PSF's sampled points in the image plane
        % spat_freq=2*pi/lambda/f_tube*im_coords;       % spatial frequencies in the Fraunhofer diffraction
        % mod_spat_freq=2*pi/lambda*mod_im_coords;                % spatial frequencies in the Fraunhofer diffraction
        % exp(-1i*spat_freq*ap_coords)=exp(-1i*mod_spat_freq*mod_ap_coords)
        
	aperture_rad_tube=NA;       % modified radius of the tube lens
        
    
    % giving vales for fluorescence lifetime and rotational correlation time
    if anisotropy_bool && strcmp(params.psf.type, 'vectorial')
        an_tau_ratio=params.psf.an_tau_ratio;
        if isscalar(an_tau_ratio) || isreal(an_tau_ratio) || ~ischar(an_tau_ratio)  % the ratio must be real scalar number
            if an_tau_ratio>=0      % the ratio must be positive
                %an_tau_ratio=tau_f/tau_r
                if an_tau_ratio==Inf
                    tau_r=0;
                    tau_f=1;
                else
                    tau_r=1;
                    tau_f=an_tau_ratio*1;
                end
            else
                error('Please give a valid value for the ratio of fluorescent lifetime and rational correlation time.')
            end
        else
            error('Please give a valid value for the ratio of fluorescent lifetime and rational correlation time.')
        end
    end
    
    c=299792456;            % speed of light in m/s
    eps_0=8.854187817620*10^-12;    % vacuum permittivity in SI units
    k=2*pi/lambda;                  % wave number in "1/m"
    h_mod=6.626070040*1E-34/(2*pi); % modified Planck's contant in SI units
    
    E2_to_photon_dens=c*eps_0/2/(h_mod*c*k);
    
    if strcmp(params.psf.type,'vectorial')
        if size(params.psf.el_field_dir,1)==3 && size(params.psf.el_field_dir,2)==1 && norm(params.psf.el_field_dir)~=0
            E=params.psf.el_field_dir/norm(params.psf.el_field_dir);
        else
            error('Please give a valid direction vector for exciting electric field.')
        end
    end
    
	numerical_calc_cont_bool=true;
        
end

% approximated beam radii in "x" and "y" directions
% "z_shift_x" and "z_shift_y" has to be inverted so the "z_pos" results the
% right Gaussian PSF size on the detector
w_x=w_0_x*sqrt(1+((z_pos-(-z_focal_x))/z_R_x)^2);
w_y=w_0_y*sqrt(1+((z_pos-(-z_focal_y))/z_R_y)^2);


% range of affected area in "x" and "y" directions
psf_range_x=2.5*w_x;
psf_range_y=2.5*w_y;
%psf_range_x=750*1E-9;
%psf_range_y=750*1E-9;
[ d_x_image, psf_pos_rem_x, idx_pix_x, idx_psf_sampling_x ] = m_gen_PSF_image_sampling_points( x_pos, psf_range_x, N_pixel_x, pixel_size, sampling_number_x, params.numcalc.pixel_sampling_fill_bool);
[ d_y_image, psf_pos_rem_y, idx_pix_y, idx_psf_sampling_y ] = m_gen_PSF_image_sampling_points( y_pos, psf_range_y, N_pixel_y, pixel_size, sampling_number_y, params.numcalc.pixel_sampling_fill_bool);

% if the molecule's PSF is out of the active area
if isempty(idx_pix_x) || isempty(idx_pix_y)
   switch psf_type
       case 'scalar'
           PSF_image{1}=zeros(length(idx_pix_x),length(idx_pix_y));
       case 'vectorial'
           if ~polarization_bool
               PSF_image{1}=zeros(length(idx_pix_x),length(idx_pix_y));
           elseif polarization_bool
               PSF_image{1}=zeros(length(idx_pix_x),length(idx_pix_y));
               PSF_image{2}=zeros(length(idx_pix_x),length(idx_pix_y));
           end

   end
   return   % stop execution and return to the parent function
end



% collecting parameters for the child functions
params.numcalc.sampling_number_x=sampling_number_x;
params.numcalc.sampling_index_x=sampling_index_x;
params.numcalc.sampling_number_y=sampling_number_y;
params.numcalc.sampling_index_y=sampling_index_y;
params.numcalc.aperture_rad_obj=aperture_rad_obj;
params.numcalc.psf_pos_rem_x=psf_pos_rem_x;
params.numcalc.psf_pos_rem_y=psf_pos_rem_y;
params.numcalc.d_x_image=d_x_image;
params.numcalc.d_y_image=d_y_image;
params.numcalc.focal_plane_dist=focal_plane_dist;
params.numcalc.z_focal_x=z_focal_x;
params.numcalc.z_focal_y=z_focal_y;
params.numcalc.aperture_rad_tube=aperture_rad_tube;
%params.numcalc.E=E;

[ NA_LG, delta_d_i ] = m_gen_PSF_aperture_func_focal_position( params );


end