function [NA_LG, delta_d_i ] = test_PSF_variance_delta_d_i(NA, lambda,  focal_plane_dist, d_s, n_s, n_i )
%UNTITLED Summary of this function goes here
%   Detailed explanation goes here


    function [x, functionValue, exitflag] = PSF_focus_iteration_custom(functionHandle, x0)
        
        % parameters for the iteration
        initialStepSize=100*1e-9;
        derivatStep=1*1e-9;
        stepCriterion=1*1e-9;
        
        % calculate the derivate at the initial "x":
        f1=functionHandle(x0-derivatStep/2);
        f2=functionHandle(x0+derivatStep/2);
        functionDerivate=(f2-f1)/derivatStep;
        initialDerivate=functionDerivate;
        initialValue=(f1+f2)/2;
        
        x=x0;
        step=initialStepSize;
        while abs(step)>stepCriterion
            
            f1=functionHandle(x-derivatStep/2);
            f2=functionHandle(x+derivatStep/2);
            
            functionDerivate=(f2-f1)/derivatStep;
            
            step=-functionDerivate/initialDerivate*initialStepSize;
            
            x=x+step;
            
        end
        
        
        f1=functionHandle(x-derivatStep/2);
        f2=functionHandle(x+derivatStep/2);
        functionValue=(f1+f2)/2;
        
        if functionValue<initialValue
            exitflag=1;
        elseif functionValue==initialValue
            exitflag=1;
        else
            exitflag=-1;
        end
        
    end

% function for varying theta angle in the immersion medium in order that
% the ray hit the sampled point on the aperture
    function F=ray_angles(theta, h_ap)
        F=(h_ap-(d_i+delta_d_i)*tan(theta)-...
            d_g*n_i*sin(theta)/n_g./sqrt(1-(n_i*sin(theta)/n_g).^2)-...
            d_s*n_i*sin(theta)/n_s./sqrt(1-(n_i*sin(theta)/n_s).^2))*1E9;
    end

    function [theta, error_ray_angles_iteration, exitflag_ray_angles_iteration]=ray_angles_iteration(theta_edge_i_0, theta_i_max_0, theta_i_min_0, h_ap)
        theta=theta_edge_i_0;
        theta_max=theta_i_max_0;        % maximal angle value, the angle belonging to the edge must be under it
        theta_min=theta_i_min_0;        % minimal angle value, the angle belonging to the edge must be over it
        exitflag_ray_angles_iteration=0;
        delta_x_ap_0=-ray_angles(theta, h_ap);  % initial position change where the ray intersects the aperture
        theta_prev=theta;   % theta of the previous iteration
        for idxTheta=1:100
            theta_prev2=theta_prev; % theta of the 2 previous iteration
            theta_prev=theta;
            delta_x_ap=-ray_angles(theta, h_ap);    % position change where the ray intersects the aperture
            if delta_x_ap<0                         % if the ray intersects the aperture below the edge
                theta_min=theta;
                theta=(5*theta_max+theta_min)/6;    % weighted mean, for faster convergence
            elseif delta_x_ap>0                     % if the ray intersects the aperture above the edge
                theta_max=theta;
                theta=(5*theta_min+theta_max)/6;    % weighted mean, for faster convergence
            elseif delta_x_ap==0                    % if the ray intersects the aperture exactly at the edge
                exitflag_ray_angles_iteration=1;
                error_ray_angles_iteration=(delta_x_ap)/h_ap;
                break;
            end
            
            % if the "theta" change is in the order of the smallest number,
            % without it, it might not converge to it, only oscillate
            % around it
            if abs(theta-theta_prev)==eps
                if abs(delta_x_ap)<=h_ap*1e-4
                    exitflag_ray_angles_iteration=1;
                    break;
                end
            elseif theta==theta_prev2
                if theta>theta_prev
                    theta=theta_prev+eps;
                elseif theta<theta_prev
                    theta=theta_prev-eps;
                end
            end
            
        end
        
        % in this case, consider the iteration successful
        if abs(delta_x_ap)<=h_ap*1e-4 
            exitflag_ray_angles_iteration=1;
        end
        error_ray_angles_iteration=(delta_x_ap)/h_ap;
    end

% function for calculating the radial dependence of PSF during PSF
% size minimalization of PSF distorted by Gibson and Lanni aberration
    function H=PSF_func(delta_d_i_iter, defocus_iter)
        H=abs((besselj(0,2*pi*2.5*0.5*ro_det_var*ro_tube_var')*(ro_tube_var.*apodizing_func_PSF_variance...           % Hankel transform of the approximated aperture function
            .*exp(1i*k*(d_s*sqrt(n_s^2-NA_LG^2*ro_tube_var.^2)+delta_d_i_iter*sqrt(n_i^2-NA_LG^2*ro_tube_var.^2)-defocus_iter*ro_tube_var.^2))))').^2;    % Gibson and Lanni aberration
    end

% function for calculating the variance of PSF during PSF
% size minimalization of PSF disrtorted by Gibson and Lanni aberration
    function I=PSF_variance(delta_d_i_iter)
        defocus_iter=0;                                 % do not add defocus arbitrarily
        I=PSF_func(delta_d_i_iter, defocus_iter)...                           % PSF
            /(PSF_func(delta_d_i_iter, defocus_iter)*2*pi*ro_det_var)...     % normalization
            *(2*pi*ro_det_var.^3)*1E3;                             % multipliing with radius^2 for variance and integrating in polar coordiantes
    end


%ray_angle_iteration_method='matlab';
ray_angle_iteration_method='custom';


% lambda=670*1E-9;
k=2*pi/lambda;   % wave number in "1/m"

% focal_plane_dist=300*1E-6;
% NA=1.20;
% d_s=5*1E-6;
% n_s=1.331;
% n_i=1.518;
d_i=focal_plane_dist;
n_g=n_i;
d_g=0;

% calculating the objective's aperture's radius from the NA, focal length
% and the refractive index of immersion medium
% the refractive index of the immersion medium and the cover slip are
% assumed to be equal
alpha_i=asin(NA/n_i);   %the maximal half-angle of the cone of light in the immersion medium

% if the angles are complex, e.g. wrong values were given either for the
% refractive indices or for the numerical aperture
if ~isreal(alpha_i)
    error('Wrong immersion medium RI for the given numerical aperture.')
end

aperture_rad_obj=focal_plane_dist*tan(alpha_i);  % radius of the objective's aperture in m

sampling_number_var=256;

ro_tube_var=(0.5:sampling_number_var-0.5)'/sampling_number_var;        % normalized radii values of the tube lens' aperture
ro_det_var=(0.5:sampling_number_var-0.5)'/sampling_number_var;         % normalized radii values of the imaged area

delta_d_i_0=-d_s*n_i/n_s;               % initial thickness change of the immersion medium
delta_d_i=delta_d_i_0;                  % thickness change of the immersion medium

test_delta_d_i=delta_d_i;       % for testing

% determining the maximal ray angles belonging to the total intrenal
% refraction in the immersion medium
switch max([n_s, n_g, n_i])
    case n_i    % if the immersion medium has the highest refractive index
        theta_i_max=asin(min([n_s, n_g, n_i])/n_i);
    case n_g    % if the cover slip has the highest refractive index
        if n_i>n_s
            theta_i_max=asin(min([n_s, n_g, n_i])/n_i);
        elseif n_i<=n_s
            theta_i_max=pi/2;
        end
    case n_s    % if the sample medium has the highest refractive index
        if n_i>n_g
            theta_i_max=asin(min([n_s, n_g, n_i])/n_i);
        elseif n_i<=n_g
            theta_i_max=pi/2;
        end
end

% initial guess for the ray angles belonging to the intersection border points
theta_edge_i_0=atan(abs(aperture_rad_obj)/(d_i+delta_d_i+d_s));     % initial guess for the ray angles, required for the iteration method

%apodizing_func=cos(atan(abs(aperture_rad_obj*ro_tube_iter)/(delta_d_i))).^0.5;

% approximated apodizing function, gives the "shape" of the aperture function
% did not want to calculate it here, did not want to run the expensive
% iteration for all ray angles here too and experienced as the the shape of
% the aperture function does not affects much the outcome variable value of
% minimalization of the PSF
apodizing_func_PSF_variance=cos(theta_edge_i_0*ro_tube_var).^1.5;           % this fits relatively well to the product of the apodizing function and the transmission coefficients (modified apodizing function)
%apodizing_func_PSF_variance=1;

theta_max_ratio=0.95;   % starting maximum allowed ray angle and angle of total internal reflection ration

for var_iter_idx=1:1        % let this the maximal number of iterations for calculating the "z" pos where the PSF variance is minimal
    var_iter_success_bool=true;    % do not break out of the "while" loop
        % running the itarition with different "theta_bord_i_0_max" values
        
        switch ray_angle_iteration_method
            case 'matlab'
                options = optimset('Display','off', 'FinDiffType', 'central');        % do not display iteration output
                N_fsolve_loop=10;       % maximal number of loops for the iteration of "theta_i_ap_edge", inner loops, should be enough
                fsolve_loop=1;       % variable for the "while" loop
                while fsolve_loop<=N_fsolve_loop
                    iter_success_bool=true;    % boolean for storing whether all the iterations were successful
                    theta_edge_i_0_max=theta_i_max*theta_max_ratio;            % let this be the maximal initial ray angle, should be smaller than the angle of total internal refraction
                    % initial ray angle for the iteration
                    if var_iter_idx==1
                        theta_edge_i_0_iter=theta_edge_i_0;
                    else
                        theta_edge_i_0_iter=theta_edge_i;
                    end
                    theta_edge_i_0_iter(theta_edge_i_0_iter>theta_edge_i_0_max)=theta_edge_i_0_max;   % if any of the initial guess exceeds the maximal value...
                    
                    [theta_edge_i, ~, exitflag]=fsolve(@(theta) ray_angles(theta, abs(aperture_rad_obj)), theta_edge_i_0_iter, options);   % iteration for getting the ray angles
                    % examining whether the iteration was successfull for a given ray
                    if exitflag>0       % if the iteration was successful
                    else                % if the iteration was not successful
                        iter_success_bool=false;    % do not break out of the "while" loop
                    end
                    
                    % if the iterations were successful for all angles...
                    if iter_success_bool
                        break;           % break out of the loop
                    end
                    % decrease the "theta_bord_i_0_max" and "theta_i_max" difference in
                    % the next loop
                    fsolve_loop=fsolve_loop+1;
                    theta_max_ratio=1-(1-theta_max_ratio)/2;
                end
            case 'custom'
                [theta_edge_i, ~, exitflag]=ray_angles_iteration(theta_edge_i_0, theta_i_max, 0, abs(aperture_rad_obj));
                % examining whether the iteration was successfull for a given ray
                if exitflag>0       % if the iteration was successful
                    iter_success_bool=true;
                else                % if the iteration was not successful
                    iter_success_bool=false;
                end
        end
                               
        
%     options = optimset('Display','off', 'FinDiffType', 'central');        % do not display iteration output
%     N_fsolve_loop=10;       % maximal number of loops for the iteration of "theta_i_ap_edge", inner loops, should be enough
%     fsolve_loop=1;       % variable for the "while" loop
%     while fsolve_loop<=N_fsolve_loop
%         iter_success_bool=true;    % boolean for storing whether all the iterations were successful
%         theta_edge_i_0_max=theta_i_max*theta_max_ratio;            % let this be the maximal initial ray angle, should be smaller than the angle of total internal refraction
%         % initial ray angle for the iteration
%         if var_iter_idx==1
%             theta_edge_i_0_iter=theta_edge_i_0;
%         else
%             theta_edge_i_0_iter=theta_edge_i;
%         end
%         theta_edge_i_0_iter(theta_edge_i_0_iter>theta_edge_i_0_max)=theta_edge_i_0_max;   % if any of the initial guess exceeds the maximal value...
%         
%         [theta_edge_i, ~, exitflag]=fsolve(@(theta) ray_angles(theta, abs(aperture_rad_obj)), theta_edge_i_0_iter, options);   % iteration for getting the ray angles
%         % examining whether the iteration was successfull for a given ray
%         if exitflag>0       % if the iteration was successful
%         else                % if the iteration was not successful
%             iter_success_bool=false;    % do not break out of the "while" loop
%         end
%             
%         % if the iterations were successful for all angles...
%         if iter_success_bool
%            break;           % break out of the loop
%         end
%         % decrease the "theta_bord_i_0_max" and "theta_i_max" difference in
%         % the next loop
%         fsolve_loop=fsolve_loop+1;
%         theta_max_ratio=1-(1-theta_max_ratio)/2;
%     end
    
    if ~iter_success_bool
        error('Could not calculate ray angle for aperture edge. Please decrease the numerical aperture.')
    end

    NA_LG=n_i*sin(theta_edge_i);    % numerical aperture in Gibson and Lanni aberration
    
    if var_iter_idx==1
       test_NA_LG=NA_LG;        % for testing
    end
    
    delta_d_i_0=delta_d_i;      % starting immersion thickness change for the PSF variance calculation
    
    %[delta_d_i,~,exitflag_var_iter]=fminsearch(@(delta_d_i_iter) PSF_variance(delta_d_i_iter), 1.3*delta_d_i_0);       % minimizing the PSF variance
    [delta_d_i,~,exitflag_var_iter]=PSF_focus_iteration_custom(@(delta_d_i_iter) PSF_variance(delta_d_i_iter), delta_d_i_0);       % minimizing the PSF variance
    
    if exitflag_var_iter>0       % if the iteration was successful
        if abs((delta_d_i-delta_d_i_0)/delta_d_i_0)<1E-4
            var_iter_success_bool=true;
            break;
        end
    else                % if the iteration was not successful
        var_iter_success_bool=false;    % do not break out of the "while" loop
        break;
    end
    
end

if ~var_iter_success_bool       % if the iteration was successful
    error('Could not minimalize the PSF variance. Please decrease the numerical aperture or the simple medium thickness.')                % if the iteration was not successful
end

a=besselj(0,2*pi*2.5*0.5*ro_det_var*ro_tube_var')*(ro_tube_var.*apodizing_func_PSF_variance);     % aberration free PSF
b=PSF_func(delta_d_i, 0);              % PSF after the iterations
NA_LG=test_NA_LG;
c=PSF_func(test_delta_d_i, 0);         % PSF with the initial values

% figure()
% plot(abs(a).^2)         % aberration free PSF
% 
% figure()
% plot(b)                 % PSF after the iterations
% 
% figure()
% plot(c)                 % PSF with the initial values
% 
% figure()
% plot(apodizing_func_PSF_variance)    % "shape" of the aperture function

% values after and before the iterations
% delta_d_i
% test_delta_d_i
% 
% NA_LG


end

