% adding the directories containing the required functions
run('../addSubfoldersToPath.m')

% Coordinates of the fluorophores:
xCoords_mol=[];
yCoords_mol=[];
zCoords_mol=[];

pos=[...
    xCoords_mol;...
    yCoords_mol;...
    zCoords_mol];

% Parameters for 3D astigmatic simulation
PSF_params.meas.astigmatic_bool=false;
PSF_params.meas.tube_f=20;      %in "cm"
PSF_params.meas.magnification=100;
PSF_params.meas.cylindrical_f=10;   % in "m"

% Parameters for the polarization sensitive simulation
PSF_params.meas.polarization_bool=false;

% Choose a PSF model
PSF_params.psf.type='gaussian';
%PSF_params.psf.type='scalar';
%PSF_params.psf.type='vectorial';

%Parameters for the Gaussian PSf model
PSF_params.psf.colleff=0.1;

% Parameters for the scalar or for the vectorial PSF model
PSF_params.psf.focal_plane_dist=300;       % distance between the focal plane and the objective's aperture in um
PSF_params.psf.sample_depth=5;
PSF_params.psf.an_cone_angle=5;
PSF_params.psf.el_field_dir=[1;1i;0];
PSF_params.psf.an_tau_ratio=Inf;
PSF_params.psf.anisotropy_bool=false;
PSF_params.psf.readApertureFuncBool=false;

% Orientation of the fluorphores, for the vectorial PSF model
PSF_params.mol.orientation(1)=45*pi/180;       % fi (azimuth) angle of orientation of the dipole
PSF_params.mol.orientation(2)=90*pi/180;       % theta (polar) angle of orientation of the dipole

% dye parameters
PSF_params.dye.em_N_ofphotonpersec=34060;
PSF_params.dye.lambda=520;

% Acquisition parameters
PSF_params.aq.frame_size=10;
PSF_params.aq.pixel_size=160;
%params.aq.NA=1.357;     % scalar PSF
%params.aq.NA=0.9;     % scalar PSF
PSF_params.aq.NA=0.961;     % Gaussian PSF
PSF_params.aq.n_immersion=1.52;
PSF_params.aq.n_sample=1.33;

PSF_params.mol.ideal_PSF_bool=false;


%% setting the parameters of the image stack

stack_params.trialsN=1000;
stack_params.background=456.9+67.49;
stack_params.PoissonNoiseBool=true;
stack_params.electronsPerCount=3.745;   % normal 1MHz
%stack_params.electronsPerCount=1.281;   % EM 10MHz


%% set the localization parameters

algo_index=3;      % multi Gaussian
%algo_index=6;       % single Gaussian

algo_list = rainSTORM_default_settings_loc_algos();

%choose recon algo:
loc_algo_name=algo_list.names{algo_index};
loc_algo_handle=algo_list.fhandle{algo_index};


loc_settings = rainSTORM_default_settings_loc_algos(loc_algo_name);

% loc_settings.initSig.val;
% loc_settings.tol.val;
% loc_settings.maxIts.val;
% loc_settings.allowSig.val;
% loc_settings.allowX.val;
% loc_settings.rad.val;


%% Parameters the separability examination:

distN=41;
trialsN=1000;

distStart=250;
distEnd=500;

center=[ 0, 0, 0];
angle=0;
%angle=45;
%angle=22.5;

%% run the seperability simlation
seperationSuccessRatio=zeros(distN,1);
distances=linspace(distStart, distEnd, distN);
for idxDist=1:distN
    
    distance=[distances(idxDist)*cos(angle*pi/180), distances(idxDist)*sin(angle*pi/180), 0];
    
    % Coordinates of the fluorophores:
    xCoords_mol=[center(1)-distance(1)/2, center(1)+distance(1)/2];
    yCoords_mol=[center(2)-distance(2)/2, center(2)+distance(2)/2];
    zCoords_mol=[center(3)-distance(3)/2, center(3)+distance(3)/2];
    
    pos=[...
    xCoords_mol;...
    yCoords_mol;...
    zCoords_mol];
    
    [frameStack] = generate_Frame(pos, PSF_params, stack_params);
    [~, locN] = localize_Frame(frameStack, loc_algo_handle, loc_settings);
    
    seperationSuccessRatio(idxDist)=sum(locN==2)/stack_params.trialsN;
end


%% visualize the seperability results

measuredData.values=seperationSuccessRatio;
measuredData.coordinates=distances;

functionToFitString='errorFunction';

functionParameters.background=1;
functionParameters.width=10;
functionParameters.center=mean(distances);
fittingParameterList={'width', 'center'};
minimizationBool=true;
constraints.width=[-Inf, Inf];
constraints.center=[-Inf, Inf];
options=[];
modelSettings=struct();
algorithmString='fminsearch';
functionalToOptimizeString='optimizationFunctionals.meanSquaredError';

[fittedParameters, FVAL, EXITFLAG] = functionFitting(measuredData, functionToFitString, modelSettings, functionParameters, fittingParameterList, constraints, algorithmString, functionalToOptimizeString, minimizationBool, options);



fittedParameters.center

distancesFiner=linspace(distStart, distEnd, 1000);
% values of the model function:
functionToFitHandle=str2func(functionToFitString);
[ sampledFunction ] = functionToFitHandle(distancesFiner, fittedParameters, modelSettings);

figure()
scatter(distances, seperationSuccessRatio, 'x')
hold on;
plot(distancesFiner, sampledFunction)

xlabel('seperation distance [nm]');
ylabel('seperation probability');

labelText={['center: ', sprintf('%.4g', fittedParameters.center), ' nm'], ['width: ', sprintf('%.4g', fittedParameters.width), ' nm']};
text([0.65, 0.65], [0.25, 0.15], labelText, 'Units', 'normalized')

hold off;


% removing the directories added for the sscript
run('../removeSubfoldersFromPath.m')
