function [frameStack] = generate_Frame(pos, PSF_params, stack_params)


% clearing the persistent variables used for PSF generation
clear m_gen_PSF_gauss % it must be cleared as frame size can change in the next simulation
% no need to clear them, just for safety reason:
clear m_gen_PSF_numerical_calc
clear m_gen_PSF_aperture_func
clear m_gen_PSF_DFT
clear m_gen_PSF_pixelise


NBlinking=size(pos,2);

frameSize=PSF_params.aq.frame_size;

Frame=zeros(frameSize,frameSize);

global Gaussian_beam_params_bool;
global DFT_matrix_bool;
global aperture_func_bool;
global numerical_calc_cont_bool

% booleans for deciding whether the persistent variables should be calculated (or recalculated)
Gaussian_beam_params_bool=false;
DFT_matrix_bool=false;
aperture_func_bool=false;
numerical_calc_cont_bool=false;

for idxBlinking=1:NBlinking

    PSF_params.mol.pos=pos(:,idxBlinking);
    
    if strcmp(PSF_params.psf.type,'vectorial') && PSF_params.psf.readApertureFuncBool
    % if the electrical field at the aperture should be read from an
    % external simulation
        PSF_params.lumerical_monitorDistance=40*1e-9;
        PSF_params.lumerical_dipoleDepth=30*sin(20*pi/180)*1e-9;
        
        PSF_params.psf.sample_depth=PSF_params.lumerical_dipoleDepth;
        
        [PSF_image, idx_pix_x, idx_pix_y]=m_gen_PSF_calc( PSF_params, true );
    else 
    % generating the PSFs completely with TestSTORM
        [PSF_image, idx_pix_x, idx_pix_y]=m_gen_PSF_calc( PSF_params, true );
    end
    
    % adding the pixelized PSF to the stack
    Frame(idx_pix_x, idx_pix_y)=Frame(idx_pix_x, idx_pix_y)+PSF_image{1};
    
    if strcmp(PSF_params.psf.type,'vectorial') && PSF_params.meas.polarization_bool
        Frame(idx_pix_x, idx_pix_y)=Frame(idx_pix_x, idx_pix_y)+PSF_image{2};
    end
    
end


Frame=Frame+stack_params.background;

Frame=repmat(Frame, [1,1,stack_params.trialsN]);

if stack_params.PoissonNoiseBool
    Frame=m_gen_random('poisson', Frame);
else
    % do nothing
end

frameStack=uint16(Frame/stack_params.electronsPerCount);


% clear the persistent variables again, just cleaning up the mess:
% clearing the persistent variables used for PSF generation
clear m_gen_PSF_gauss % it must be cleared as frame size can change in the next simulation
% no need to clear them, just for safety reason:
clear m_gen_PSF_numerical_calc
clear m_gen_PSF_aperture_func
clear m_gen_PSF_DFT
clear m_gen_PSF_pixelise

end

