% adding the directories containing the required functions
run('../addSubfoldersToPath.m')

% This script is suitable for testing the PSFs generated by TestSTORM.

filename=mfilename('fullpath');
[m_gen_path] = fileparts(filename);

addpath([m_gen_path,filesep,'functions'])

%% Parameters to set:
% Coordinates of the fluorophores [nm]:
xCoords_mol=[0];
yCoords_mol=[0];
zCoords_mol=[0];

pos=[...
    xCoords_mol;...
    yCoords_mol;...
    zCoords_mol];

% Parameters for 3D astigmatic simulation
PSF_params.meas.astigmatic_bool=false;
PSF_params.meas.tube_f=20;      %in "cm"
PSF_params.meas.magnification=100;
PSF_params.meas.cylindrical_f=10;   % in "m"

% Choose a PSF model
PSF_params.psf.type='gaussian';
%PSF_params.psf.type='scalar';
%PSF_params.psf.type='vectorial';
% Parameters for the polarization sensitive simulation
PSF_params.meas.polarization_bool=false;

if PSF_params.meas.polarization_bool && ~strcmp(PSF_params.psf.type, 'vectorial')
   error('Polarization sensitive simulation can only be performed with the vectorial PSF model.') 
end

%Parameters for the Gaussian PSf model
PSF_params.psf.colleff=0.3;

% Parameters for the scalar or for the vectorial PSF model
PSF_params.psf.focal_plane_dist=300;       % distance between the focal plane and the objective's aperture in um
PSF_params.psf.sample_depth=5;
PSF_params.psf.an_cone_angle=5;
PSF_params.psf.el_field_dir=[1;0;1i];
PSF_params.psf.an_tau_ratio=Inf;
PSF_params.psf.anisotropy_bool=false;
PSF_params.psf.readApertureFuncBool=true;

% Orientation of the fluorphores, for the vectorial PSF model
PSF_params.mol.orientation(1)=0*pi/180;       % fi (azimuth) angle of orientation of the dipole
PSF_params.mol.orientation(2)=0*pi/180;       % theta (polar) angle of orientation of the dipole

% dye parameters
PSF_params.dye.em_N_ofphotonpersec=5000;
PSF_params.dye.lambda=665;

% Acquisition parameters
PSF_params.aq.frame_size=10;
PSF_params.aq.pixel_size=160;
PSF_params.aq.NA=1.4;
PSF_params.aq.n_immersion=1.52;
PSF_params.aq.n_sample=1.33;

PSF_params.mol.ideal_PSF_bool=false;

%% setting the parameters of the image stack

stack_params.trialsN=1;
stack_params.background=0;
stack_params.PoissonNoiseBool=false;
stack_params.electronsPerCount=1;


%% frame (single) generation

[Frame] = generate_Frame(pos, PSF_params, stack_params);

%% displaying results
figure()
image(Frame(:,:,1)'/(max(max(Frame(:,:,1)))/64))
axis square
title('Pixelized PSF')



% removing the directories added for the sscript
run('../removeSubfoldersFromPath.m')