% adding the directories containing the required functions
run('../addSubfoldersToPath.m')

% This script is suitable for testing the PSFs generated by TestSTORM and
% for testing the localization algorithms of rainSTORM.

%% Parameters to set:
% Coordinates of the fluorophores:
xCoords_mol=[0, 0];
yCoords_mol=[0, 0];
zCoords_mol=[0, 0];

pos=[...
    xCoords_mol;...
    yCoords_mol;...
    zCoords_mol];

% Parameters for 3D astigmatic simulation
PSF_params.meas.astigmatic_bool=false;
PSF_params.meas.tube_f=20;      %in "cm"
PSF_params.meas.magnification=100;
PSF_params.meas.cylindrical_f=10;   % in "m"

% Parameters for the polarization sensitive simulation
PSF_params.meas.polarization_bool=true;

% Choose a PSF model
%PSF_params.psf.type='gaussian';
%PSF_params.psf.type='scalar';
PSF_params.psf.type='vectorial';

%Parameters for the Gaussian PSf model
PSF_params.psf.colleff=0.3;

% Parameters for the scalar ot for the vectorial PSF model
PSF_params.psf.focal_plane_dist=300;       % distance between the focal plane and the objective's aperture in um
PSF_params.psf.sample_depth=5;
PSF_params.psf.an_cone_angle=90;
PSF_params.psf.el_field_dir=[0;1;0];
PSF_params.psf.an_tau_ratio=8.84;
PSF_params.psf.anisotropy_bool=true;
PSF_params.psf.readApertureFuncBool=true;

% Orientation of the fluorphores, for the vectorial PSF model
PSF_params.mol.orientation(1)=0*pi/180;       % fi (azimuth) angle of orientation of the dipole
PSF_params.mol.orientation(2)=0*pi/180;       % theta (polar) angle of orientation of the dipole

% dye parameters
PSF_params.dye.em_N_ofphotonpersec=5000;
PSF_params.dye.lambda=520;

% Acquisition parameters
PSF_params.aq.frame_size=10;
PSF_params.aq.pixel_size=160;
PSF_params.aq.NA=1.40;
PSF_params.aq.n_immersion=1.52;
PSF_params.aq.n_sample=1.33;

PSF_params.mol.ideal_PSF_bool=false;

%% setting the parameters of the image stack

stack_params.trialsN=1;
stack_params.background=0;
stack_params.PoissonNoiseBool=false;
stack_params.electronsPerCount=1;

%% frame (single) generation

[Frame] = generate_Frame(pos, PSF_params, stack_params);

%% displaying results
figure()
image(Frame(:,:,1)'/max(max(Frame(:,:,1)))*64)
axis square
title('Pixelized PSF')

%% set the localization parameters

algo_index=3;

algo_list = rainSTORM_default_settings_loc_algos();

%choose recon algo:
loc_algo_name=algo_list.names{algo_index};
loc_algo_handle=algo_list.fhandle{algo_index};


loc_settings = rainSTORM_default_settings_loc_algos(loc_algo_name);

% loc_settings.initSig.val;
% loc_settings.tol.val;
% loc_settings.maxIts.val;
% loc_settings.allowSig.val;
% loc_settings.allowX.val;
% loc_settings.rad.val;


%% localization
[SupResParams,~] = localize_Frame(Frame, localgo_handle, loc_settings);


xCoords_loc=([SupResParams(:).x_coord]'-frameSize/2)*PSF_params.aq.pixel_size;
yCoords_loc=([SupResParams(:).y_coord]'-frameSize/2)*PSF_params.aq.pixel_size;

distances=sqrt((xCoords_loc-xCoords_mol).^2+(yCoords_loc-yCoords_mol).^2);

%% displaying results

fprintf(['N of PSFs/N of locs:\t\t' , num2str([length(xCoords_mol),length(xCoords_loc)]), '\n'])
fprintf(['Error in the distances:\t\t' , num2str(min(distances,[],1)), '\n'])

fprintf(['Row residuum:\t\t\t\' , num2str([SupResParams.res_Row]), '\n'])
fprintf(['Col residuum:\t\t\t\t' , num2str([SupResParams.res_Col]), '\n'])

% coordinates of the localization
[xCoords_loc, yCoords_loc]


% removing the directories added for the sscript
run('../removeSubfoldersFromPath.m')